/***************************************************************************

     File: xmtutor.c
 Contents: Motif tutorial, main program (creates dynamic widget hierarchy)
  Part of: The Xmtutor application
   Author: Jan Borchers (job@ira.uka.de)

  $Source: /home/mit1/job/distribution/Xmtut/RCS/xmtutor.c,v $
$Revision: 1.4 $
    $Date: 1994/10/19 10:29:46 $
   $State: Exp $
  $Locker:  $

$Log: xmtutor.c,v $
 * Revision 1.4  1994/10/19  10:29:46  job
 * Changed debugging printf call (not normally used) to avoid
 * compilation warning (changed %x to %lu in format string).
 *
 * Revision 1.3  1994/10/17  18:10:01  job
 * First revision under version control.
 *

***************************************************************************/


/**************************************************************************/
/*                                                                        */
/* COPYRIGHT NOTICE:                                                      */
/* -----------------                                                      */
/* All parts of the Xmtutor application and the sample Motif tutorial are */
/* Copyright (C) 1993,1994 Jan Borchers.                                  */
/*                                                                        */
/**************************************************************************/


#include <stdio.h>		/* printf, etc. (FOR DEVELOPMENT ONLY) */
#include <fcntl.h>		/* File handling */

#include <X11/Intrinsic.h>	/* Xt structures & variables */
#include <X11/StringDefs.h>	/* Standard Xt string constant definitions */
#include <Xm/Xm.h>		/* Common Motif variables */
#include <Xm/XmP.h>		/* Private Motif variables, for ups debugging and XtResizeWidget()*/
#include <Xm/MainW.h>		/* MainWindow */
#include <Xm/RowColumn.h>	/* MenuBar, MenuPane */
#include <Xm/Form.h>		/* Form */
#include <Xm/List.h>		/* List */
#include <Xm/CascadeB.h>	/* CascadeButton */
#include <Xm/PushB.h>		/* PushButton */
#include <Xm/Label.h>		/* Label */
#include <Xm/PanedW.h>		/* PanedWindow */
#include <Xm/Text.h>		/* Text */
#include <Xm/MessageB.h>	/* ErrorDialog */
#include <Xm/SelectioB.h>	/* SelectionBox */
#include <Xm/ScrolledW.h>	/* ScrolledWindow */

#include "IMPORT.H"		/* IMPORT/EXPORT mechanism: Imports the following identifiers as "extern". */
#include "widgets.h"            /* Dynamic widget tree hierarchy. */
#include "typeconverter.h"	/* String to Cardinal type converter for the resource manager. */
#include "sectionio.h"
#include "runio.h"		/* For SetUserResourcePath () */
#include "init.h"		/* For GetXmtutorResources (), InitIndex () */
#include "callbacks.h"          /* Callback functions */

XtAppContext app_con;
XEvent event;			/* for debugging */


void do_events()		/* This function can replace XtAppMainLoop() for debugging. */

{
  XFlush(XtDisplay(toplevel));
  while(1) {			/* better than "while (XtAppPending(app_con))" */
    XtAppNextEvent(app_con,&event);
    printf ("Next event type: %d\n", event.type);
    printf ("Next event window: %lu\n", event.xany.window);
    /* getchar(); */
    XtDispatchEvent(&event);
    XFlush(XtDisplay(toplevel));
  }
}


void main(int argc, char *argv[])
{
  
  Arg args [10];			 /* Argument list for widgets */
  Cardinal nargs;				 /* number of arguments */
  
  toplevel = XtAppInitialize(&app_con,"Xmtutor",NULL,0,&argc,argv,NULL,NULL,0);
  
  XtAddConverter (XtRString, XtRCardinal, (void *) StringToCardinal, NULL, 0); /* Register additional type converter */
  GetXmtutorResources(toplevel);				/* Get information on chapters & sections from resource file */
  
  /* if POSSIBLE, MOVE THIS HARDCODED GEOMETRY INTO THE Xmtutor RESOURCE FILE! */
  nargs = 0;
  XtSetArg (args[nargs], XmNgeometry, "1000x720");  nargs++;
  XtSetValues (toplevel, args, nargs);

  mainwindow = XmCreateMainWindow (toplevel, "mainwindow", NULL, 0);		/* Create a MainWindow as its child */

  XtManageChild (mainwindow);
  
  menubar = XmCreateMenuBar (mainwindow, "menubar", NULL, 0);			/* Create a MenuBar */
  XtManageChild (menubar);

  filemenu = XmCreatePulldownMenu (menubar, "filemenu", NULL, 0);		/* Create the "File" menu tree */

  nargs = 0;
  XtSetArg (args[nargs], XmNsubMenuId, filemenu);  nargs++;
  filecascade = XmCreateCascadeButton (menubar, "filecascade", args, nargs);
  XtManageChild (filecascade);

  restorebutton = XmCreatePushButton (filemenu, "restorebutton", NULL, 0);
  XtManageChild (restorebutton);
  
  exitbutton = XmCreatePushButton (filemenu, "exitbutton", NULL, 0);
  XtManageChild (exitbutton);

  exitdialog = XmCreateQuestionDialog (exitbutton, "exitdialog", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (exitdialog, XmDIALOG_HELP_BUTTON));
  
  viewmenu = XmCreatePulldownMenu (menubar, "viewmenu", NULL, 0);		/* Create the "View" menu tree */
  nargs = 0;
  XtSetArg (args[nargs], XmNsubMenuId, viewmenu);  nargs++;

  viewcascade = XmCreateCascadeButton (menubar,"viewcascade", args,nargs);
  XtManageChild (viewcascade);

  nextsectionbutton = XmCreatePushButton (viewmenu, "nextsectionbutton", NULL, 0);
  XtManageChild (nextsectionbutton);

  lastsectionerror = XmCreateErrorDialog (nextsectionbutton, "lastsectionerror", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (lastsectionerror, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (lastsectionerror, XmDIALOG_HELP_BUTTON));

  previoussectionbutton = XmCreatePushButton (viewmenu, "previoussectionbutton", NULL, 0);
  XtManageChild (previoussectionbutton);

  firstsectionerror = XmCreateErrorDialog (previoussectionbutton, "firstsectionerror", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (firstsectionerror, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (firstsectionerror, XmDIALOG_HELP_BUTTON));

  choosesectionbutton = XmCreatePushButton (viewmenu, "choosesectionbutton", NULL, 0);
  XtManageChild (choosesectionbutton);

  choosesectiondialog = XmCreateFormDialog (choosesectionbutton, "choosesectiondialog", NULL, 0);

  choosechapterlabel = XmCreateLabel (choosesectiondialog, "choosechapterlabel", NULL, 0);
  XtManageChild (choosechapterlabel);

  choosesectionlabel = XmCreateLabel (choosesectiondialog, "choosesectionlabel", NULL, 0);
  XtManageChild (choosesectionlabel);

  choosechapterwindow = XmCreateScrolledWindow (choosesectiondialog, "choosechapterwindow", NULL, 0);
  XtManageChild (choosechapterwindow);

  choosechapterlist = XmCreateList (choosechapterwindow, "choosechapterlist", NULL, 0);
  XtManageChild (choosechapterlist);

  choosesectionwindow = XmCreateScrolledWindow (choosesectiondialog, "choosesectionwindow", NULL, 0);
  XtManageChild (choosesectionwindow);

  choosesectionlist = XmCreateList (choosesectionwindow, "choosesectionlist", NULL, 0);
  XtManageChild (choosesectionlist);

  chooseokbutton = XmCreatePushButton (choosesectiondialog, "chooseokbutton", NULL, 0);
  XtManageChild (chooseokbutton);

  choosecancelbutton = XmCreatePushButton (choosesectiondialog, "choosecancelbutton", NULL, 0);
  XtManageChild (choosecancelbutton);

  indexbutton = XmCreatePushButton (viewmenu, "indexbutton", NULL, 0);
  XtManageChild (indexbutton);

  indexdialog = XmCreateSelectionDialog (indexbutton, "indexdialog", NULL, 0);
  XtUnmanageChild (XmSelectionBoxGetChild (indexdialog, XmDIALOG_APPLY_BUTTON));
  XtUnmanageChild (XmSelectionBoxGetChild (indexdialog, XmDIALOG_HELP_BUTTON));

  nosuchindexerror = XmCreateErrorDialog (indexdialog, "nosuchindexerror", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (nosuchindexerror, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (nosuchindexerror, XmDIALOG_HELP_BUTTON));
		 
  runmenu = XmCreatePulldownMenu (menubar, "runmenu", NULL, 0);			/* Create the "Run" menu tree */

  nargs = 0;
  XtSetArg (args[nargs], XmNsubMenuId, runmenu);  nargs++;
  runcascade = XmCreateCascadeButton (menubar, "runcascade", args, nargs);
  XtManageChild (runcascade);

  runbutton = XmCreatePushButton (runmenu, "runbutton", NULL, 0);
  XtManageChild (runbutton);

  helpmenu = XmCreatePulldownMenu (menubar, "helpmenu", NULL, 0);		/* Create the "Help" menu tree */

  nargs = 0;
  XtSetArg (args[nargs], XmNsubMenuId, helpmenu);  nargs++;
  helpcascade = XmCreateCascadeButton (menubar, "helpcascade", args, nargs);
  XtManageChild (helpcascade);

  nargs = 0;
  XtSetArg (args[nargs], XmNmenuHelpWidget, (XtArgVal) helpcascade);  nargs++;  /* Set the menubar's HelpWidget resource */
  XtSetValues (menubar, args, nargs);

  onmenusbutton = XmCreatePushButton (helpmenu, "onmenusbutton", NULL, 0);
  XtManageChild (onmenusbutton);

  onmenusdialog = XmCreateInformationDialog (onmenusbutton, "onmenusdialog", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (onmenusdialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (onmenusdialog, XmDIALOG_HELP_BUTTON));
  
  onversionbutton = XmCreatePushButton (helpmenu, "onversionbutton", NULL, 0);
  XtManageChild (onversionbutton);

  onversiondialog = XmCreateInformationDialog (onversionbutton, "onversiondialog", NULL, 0);
  XtUnmanageChild (XmMessageBoxGetChild (onversiondialog, XmDIALOG_CANCEL_BUTTON));
  XtUnmanageChild (XmMessageBoxGetChild (onversiondialog, XmDIALOG_HELP_BUTTON));

  /* Create the workarea with information display and windows to display text, program, resources & output */
  workarea = XmCreateForm (mainwindow, "workarea", NULL, 0);
  XtManageChild (workarea);

  infodisplay = XmCreateForm (workarea, "infodisplay", NULL, 0);
  XtVaSetValues (infodisplay,
		 XmNtopAttachment, XmATTACH_FORM,
		 XmNleftAttachment, XmATTACH_FORM,
		 XmNrightAttachment, XmATTACH_FORM,
		 NULL);
  XtManageChild (infodisplay);
  
  chapterdisplay = XmCreateLabel (infodisplay, "chapterdisplay", NULL, 0);
  XtManageChild (chapterdisplay);

  sectiondisplay = XmCreateLabel (infodisplay, "sectiondisplay", NULL, 0);
  XtManageChild (sectiondisplay);

  panedwindow = XtVaCreateManagedWidget ("panedwindow", xmPanedWindowWidgetClass, workarea,
					 XmNtopAttachment,    XmATTACH_WIDGET,
					 XmNtopWidget,        infodisplay,
					 XmNbottomAttachment, XmATTACH_FORM,
					 XmNleftAttachment,   XmATTACH_POSITION,
					 XmNleftPosition,     50,
					 XmNrightAttachment,  XmATTACH_FORM,
					 NULL);
  
  programwindow = XmCreateScrolledText (panedwindow, "programwindow", NULL, 0);
  XtManageChild (programwindow);

  resourcewindow = XmCreateScrolledText (panedwindow, "resourcewindow", NULL, 0);
  XtManageChild (resourcewindow);

  textwindow = XmCreateScrolledText (workarea, "textwindow", NULL, 0);
  XtVaSetValues (XtParent (textwindow),
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, infodisplay,
		 XmNbottomAttachment, XmATTACH_FORM,
		 XmNleftAttachment, XmATTACH_FORM,
		 XmNrightAttachment, XmATTACH_POSITION,
		 XmNrightPosition, 49,
		 NULL);
  XtManageChild (textwindow);

  /* Set the areas of the MainWindow to their respective widgets */
  XmMainWindowSetAreas (mainwindow, menubar, NULL, NULL, NULL, workarea);
  
  XtAddCallback (restorebutton, XmNactivateCallback, (void *) RestoreCB, NULL);	                  /* Add callbacks */
  XtAddCallback (exitbutton, XmNactivateCallback, (void *) ExitCB, NULL);	
  XtAddCallback (exitdialog, XmNokCallback, (void *) ExitOKCB, NULL);	
  XtAddCallback (nextsectionbutton, XmNactivateCallback, (void *) NextSectionCB, NULL);
  XtAddCallback (previoussectionbutton, XmNactivateCallback, (void *) PreviousSectionCB,NULL);

  XtAddCallback (choosesectionbutton, XmNactivateCallback, (void *) ChooseSectionCB, NULL);
  XtAddCallback (choosechapterlist, XmNbrowseSelectionCallback, (void *) SetSectionListCB, NULL);
  XtAddCallback (choosechapterlist, XmNdefaultActionCallback, (void *) ChooseOKCB, NULL);
  XtAddCallback (choosesectionlist, XmNdefaultActionCallback, (void *) ChooseOKCB, NULL);
  XtAddCallback (chooseokbutton, XmNactivateCallback, (void *) ChooseOKCB, NULL);
  XtAddCallback (choosecancelbutton, XmNactivateCallback, (void *) ChooseCancelCB, NULL);

  XtAddCallback (indexbutton, XmNactivateCallback, (void *) IndexCB, NULL);
  XtAddCallback (indexdialog, XmNokCallback, (void *) IndexOKCB, NULL);
  XtAddCallback (indexdialog, XmNcancelCallback, (void *) IndexCancelCB, NULL);
  XtAddCallback (indexdialog, XmNnoMatchCallback, (void *) IndexNoMatchCB, NULL);
  XtAddCallback (runbutton, XmNactivateCallback, (void *) RunCB, NULL);
  XtAddCallback (onmenusbutton, XmNactivateCallback, (void *) OnMenusCB, NULL);
  XtAddCallback (onversionbutton, XmNactivateCallback, (void *) OnVersionCB, NULL);
   
  if (numberOfChapters == 0)					/* No chapters available -> no running, viewing, etc. */
    {
      XtSetSensitive (restorebutton, FALSE);
      XtSetSensitive (viewcascade, FALSE);
      XtSetSensitive (runcascade, FALSE);
      XtSetSensitive (panedwindow, FALSE);
      XtSetSensitive (textwindow, FALSE);
    }
  else
    {
      InitIndex (indexdialog);					/* Initialize the index dialog list */
      currentChapter = 0;					/* Display first chapter, first section */
      currentSection = 0;						
      ReadSection ();
    }
  XtRealizeWidget (toplevel);					/* Realize toplevel widget */
  /*  printf ("%u Chapters found.\n", numberOfChapters); */
  XtResizeWidget (toplevel, 1000, 720, 0);			/* This is dirty, but workarea form */
  								/* doesn't show textwindow otherwise */
  XtAppMainLoop (app_con);	                                /* Enter main event loop */
}				                                /* Use do_events() instead for debugging */

